@@ -6,6 +6,8 @@ import datetime
 
 from oscar.apps.payment.exceptions import GatewayError
 
+from . import the3rdman, xmlutils
+
 logger = logging.getLogger('datacash')
 
 # Methods
@@ -77,10 +79,11 @@ class Response(object):
 
 class Gateway(object):
 
-    def __init__(self, host, client, password, cv2avs=False, capturemethod='ecomm'):
+    def __init__(self, host, path,  client, password, cv2avs=False, capturemethod='ecomm'):
         if host.startswith('http'):
             raise RuntimeError("DATACASH_HOST should not include http")
         self._host = host
+        self._path = path
         self._client = client
         self._password = password
         self._cv2avs = cv2avs
@@ -91,7 +94,7 @@ class Gateway(object):
         conn = httplib.HTTPSConnection(self._host, 443, timeout=30)
         headers = {"Content-type": "application/xml",
                    "Accept": ""}
-        conn.request("POST", "/Transaction", request_xml, headers)
+        conn.request("POST", self._path, request_xml.encode('utf8'), headers)
         response = conn.getresponse()
         response_xml = response.read()
         if response.status != httplib.OK:
@@ -150,26 +153,43 @@ class Gateway(object):
         # TxnDetails
         txn_details = self._create_element(doc, txn, 'TxnDetails')
         if 'merchant_reference' in kwargs:
-            self._create_element(doc, txn_details, 'merchantreference', kwargs['merchant_reference'])
+            self._create_element(
+                doc, txn_details, 'merchantreference',
+                kwargs['merchant_reference'])
         if 'amount' in kwargs:
             if is_historic:
-                self._create_element(doc, txn_details, 'amount', str(kwargs['amount']))
+                self._create_element(doc, txn_details, 'amount',
+                                     str(kwargs['amount']))
             else:
-                self._create_element(doc, txn_details, 'amount', str(kwargs['amount']), {'currency': kwargs['currency']})
-        self._create_element(doc, txn_details, 'capturemethod', self._capturemethod)
+                self._create_element(
+                    doc, txn_details, 'amount', str(kwargs['amount']),
+                    {'currency': kwargs['currency']})
+        self._create_element(
+            doc, txn_details, 'capturemethod', self._capturemethod)
+
+        # The3rdMan
+        if 'the3rdman_data' in kwargs and kwargs['the3rdman_data']:
+            the3rdman.add_fraud_fields(
+                doc, txn_details, **kwargs['the3rdman_data'])
 
         return doc.toxml()
 
     def _do_request(self, method, **kwargs):
         amount = kwargs.get('amount', '')
         merchant_ref = kwargs.get('merchant_reference', '')
-        logger.info("Performing %s request - amount: %s, merchant_ref: %s" % (method, amount, merchant_ref))
+        logger.info("Performing %s request - amount: %s, merchant_ref: %s" % (
+            method, amount, merchant_ref))
 
         request_xml = self._build_request_xml(method, **kwargs)
         response_xml = self._fetch_response_xml(request_xml)
         logger.debug("Received response:\n %s" % response_xml)
 
-        return Response(request_xml, response_xml)
+        response = Response(request_xml, response_xml)
+        if response.is_successful():
+            logger.info("Response successful")
+        else:
+            logger.warning("Response unsuccessful")
+        return response
 
     def _add_cv2avs_elements(self, doc, card, kwargs):
         """
@@ -182,22 +202,17 @@ class Gateway(object):
             if key in kwargs:
                 self._create_element(doc, cv2avs, 'street_address%d' % n, kwargs[key])
         if 'postcode' in kwargs:
-            self._create_element(doc, cv2avs, 'postcode', kwargs['postcode'])
-        if 'cvv' in kwargs:
-            self._create_element(doc, cv2avs, 'cv2', kwargs['cvv'])
+            # Restrict size of postcode submitted
+            self._create_element(doc, cv2avs, 'postcode',
+                                 kwargs['postcode'][:9])
+        if 'ccv' in kwargs:
+            self._create_element(doc, cv2avs, 'cv2', kwargs['ccv'])
 
     def _create_element(self, doc, parent, tag, value=None, attributes=None):
         """
         Creates an XML element
         """
-        ele = doc.createElement(tag)
-        parent.appendChild(ele)
-        if value:
-            text = doc.createTextNode(str(value))
-            ele.appendChild(text)
-        if attributes:
-            [ele.setAttribute(k, v) for k,v in attributes.items()]
-        return ele
+        return xmlutils.create_element(doc, parent, tag, value, attributes)
 
     def _check_kwargs(self, kwargs, required_keys):
         for key in required_keys:
